#ifndef __GUI_H__
#define __GUI_H__

/***********************************************************
 * GUI MANAGEMENT
 * All GUI dialogs used throughout the game are defined
 * and handled here.
 ***********************************************************/

#include <OgreCEGUIRenderer.h>
#include <CEGUI.h>
#include <Ogre.h>
#include <OgreEventListeners.h>

class GameLogic;

class GUI: public Ogre::KeyListener, public Ogre::MouseListener, public Ogre::MouseMotionListener
{
  public:
    GUI(Ogre::RenderWindow* window, Ogre::SceneManager* sceneMgr, GameLogic* logic);
    ~GUI();

    // show the different dialogs.
    void showMainMenu();
    void showConfirmDialog();
    void showHighscore(int pos);
    void showNameDialog(int score);

    // mouse motions
    void mouseMoved(Ogre::MouseEvent* e);
    void mouseDragged(Ogre::MouseEvent* e);

    // mouse buttons
    void mouseClicked(Ogre::MouseEvent* e) { }
    void mouseEntered(Ogre::MouseEvent* e) { }
    void mouseExited(Ogre::MouseEvent* e) { }
    void mousePressed(Ogre::MouseEvent* e);
    void mouseReleased(Ogre::MouseEvent* e);

    // keys
    void keyClicked(Ogre::KeyEvent* e) { }
    void keyPressed(Ogre::KeyEvent* e);
    void keyReleased(Ogre::KeyEvent* e);

    // reactions
    bool onNewGame(const CEGUI::EventArgs& e);
    bool onHighscore(const CEGUI::EventArgs& e);
    bool onQuit(const CEGUI::EventArgs& e);
    bool onConfirmYes(const CEGUI::EventArgs& e);
    bool onConfirmNo(const CEGUI::EventArgs& e);
    bool onHighscoreClose(const CEGUI::EventArgs& e);
    bool onNameEntered(const CEGUI::EventArgs& e);

  private:
    CEGUI::MouseButton ConvertOgreButtonToCegui(int buttonID);

    CEGUI::OgreCEGUIRenderer* mGUIRenderer;
    CEGUI::System* mGUISystem;
    CEGUI::DefaultWindow* mRootWindow;
    GameLogic* mLogic;
    int mScore;
};


#endif
