#include "highscore.h"
#include <fstream>
#include <algorithm>
#include <boost/cstdint.hpp>
using namespace std;
using namespace boost;

Highscore::Highscore(const string& filename) : mFilename(filename)
{
  // read highscores from given file.
  ifstream f (filename.c_str(), ios_base::binary);
  while (f.is_open() && !f.eof())
  {
    int32_t score;
    f.read(reinterpret_cast<char*>(&score), 4);
    if (f.eof())
      break;
    int nameLen = static_cast<int>(f.get());
    char name[nameLen+1];
    name[nameLen] = 0;
    f.read(name, nameLen);
    Entry entry;
    entry.score = score;
    entry.name = string(name);
    mScoreList.push_back(entry);
  }
  sort(mScoreList.begin(), mScoreList.end(), greater<Entry>());
}

Highscore::~Highscore()
{
  // write highscores back to file.
  ofstream f (mFilename.c_str(), ios_base::binary);
  for (size_t i = 0; i < mScoreList.size(); ++i)
  {
    int32_t score = mScoreList[i].score;
    f.write(reinterpret_cast<const char*>(&score), 4);
    f.put(char(mScoreList[i].name.size()));
    f.write(mScoreList[i].name.c_str(), mScoreList[i].name.size());
  }
}

size_t Highscore::numEntries() const
{
  return mScoreList.size();
}

int Highscore::getScore(int rank) const
{
  return mScoreList[rank].score;
}

const string& Highscore::getName(int rank) const
{
  return mScoreList[rank].name;
}

int Highscore::add(int score, const string& name)
{
  Entry entry;
  entry.score = score;
  entry.name = name;
  vector<Entry>::iterator insertPos = upper_bound(mScoreList.begin(), mScoreList.end(), entry, greater<Entry>());
  int pos = distance(mScoreList.begin(), insertPos) - 1;
  mScoreList.insert(insertPos, entry);
  return pos;
}
