#ifndef __LOGIC_H__
#define __LOGIC_H__

/****************************************************
 * GAME LOGIC
 * This is the game's core. It manages the game's
 * logic in its entirety.
 ****************************************************/

#include "ExampleApplication.h"
#include "MovableText.h"
#include "highscore.h"
#include <OgreTextAreaOverlayElement.h>
#include <vector>
#include <audiere.h>

// forward declarations
class Player;
class Mouse;
class Needle;
class GUI;
class Combinations;

struct SpawnInfo
{
  double countdown;
  int colour;
};

class GameLogic : public ExampleFrameListener, public MouseMotionListener, public MouseListener
{
  bool mRunning, mStarted, mQuit;
  Player* mPlayer;
  SceneManager* mSceneMgr;
  double mTimeToNextMouse;
  bool mPrepareThrow;
  double mThrowSpeed;
  double mGameTime;
  int mScore;
  MovableText* mScoreDisplay;
  AnimationState* mCatAnim;
  GUI* mGUI;
  std::vector<SpawnInfo> mSpawnList;
  Combinations* mCombos;
  String mLastHits;
  size_t mHitsChain;
  double mShowNotice;
  Highscore mHighscore;
  
  audiere::AudioDevicePtr mAudioDevice;
  audiere::OutputStreamPtr mBkgMusic;
  audiere::OutputStreamPtr mSoundCombo;
  audiere::OutputStreamPtr mSoundCat;

  void selectCatAnimation();
  void updateCatAnimation(double passedTime);
  void initHUDElements();
  void resetHUDElements();
  void updateClockDisplay();

  void addToSpawnList(int n);
  void updateSpawnList(double passedTime);
  void updateHitDisplay();

  public:
    GameLogic(RenderWindow* win, Camera* cam, SceneManager* sceneMgr);
    ~GameLogic();

    void startGame();
    void quit();
    void pauseGame();
    void continueGame();
    void endGame(bool highscore);

    void showMainMenu();

    void updateScore(int addScore);
    void notifyHit(int colour);

    bool frameStarted(const FrameEvent& evt);

    // mouse motions
    void mouseMoved(MouseEvent* e) { }
    void mouseDragged(MouseEvent* e) { }

    // mouse buttons
    void mouseClicked(MouseEvent* e) { }
    void mouseEntered(MouseEvent* e) { }
    void mouseExited(MouseEvent* e) { }
    void mousePressed(MouseEvent* e) { }
    void mouseReleased(MouseEvent* e) { }

    // keys
    void keyClicked(KeyEvent* e) { }
    void keyPressed(KeyEvent* e);
    void keyReleased(KeyEvent* e);

    Highscore& getHighscore();
};

#endif
