#include <Ogre.h>
#include <ExampleApplication.h>
#include <ctime>
#include <cstdlib>
#include "dotscene.h"

#include "logic.h"

class DartApp : public ExampleApplication
{
  protected:
    void createScene(void)
    {
      // set shadow mode
      mSceneMgr->setShadowTechnique(SHADOWTYPE_STENCIL_ADDITIVE);
      mSceneMgr->setShadowFarDistance(1000.0);

      // load level
      Ogre::DotSceneLoader loader;
      loader.parseDotScene("Scene.xml", "General", mSceneMgr);
      // position camera
      SceneNode* camNode = mSceneMgr->getSceneNode("Camera");
      mCamera->setPosition(Vector3(0,0,0));
      camNode->attachObject(mCamera);

      // disable shadow casting for the carpet
      mSceneMgr->getEntity("Carpet")->setCastShadows(false);

       // start game
      gameLogic = new GameLogic(mWindow, mCamera, mSceneMgr);
      gameLogic->showMainMenu();

    }

    void createFrameListener(void)
    {
      mFrameListener = gameLogic;
      mRoot->addFrameListener(mFrameListener);
      mFrameListener->showDebugOverlay(false);
    }

  private:
    GameLogic* gameLogic;
};


// ----------------------------------------------------------------------------
// Main function
// ----------------------------------------------------------------------------
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT )
#else
int main(int argc, char **argv)
#endif
{
  //std::srand(std::time(0));
  DartApp app;
  try
  {
    app.go();
  }
  catch( Exception& e )
  {
#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
    MessageBox( NULL, e.getFullDescription().c_str(), "An exception has occured!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else

    std::cerr << "An exception has occured: " << e.getFullDescription();
#endif
  }
  return 0;
}
