#ifndef __MOUSE_H__
#define __MOUSE_H__

/****************************************************************
 * FLOATING MOUSE HANDLING
 * This class handles a single mouse beginning from its spawning
 * point to the starting location and during the flying phase.
 ****************************************************************/

#include <Ogre.h>
#include <audiere.h>
#include "object.h"
#include "logic.h"

class Mouse : public Ogre::UserDefinedObject, public Object
{
  public:
    Mouse(Ogre::SceneManager* mSceneMgr, GameLogic* logic, int colour, audiere::AudioDevicePtr audioDev);
    ~Mouse();

    void update(double passedTime);
    bool isDead() const;

    void show();
    void hide();

    void killBallon();

  private:
    void stateMoving(double passedTime);
    void statePreparing(double passedTime);
    void stateAir(double passedTime, bool flying);
    void stateCrashed(double passedTime);
    void stateWalkAway(double passedTime);

    enum
    {
      STATE_MOVING = 0,
      STATE_PREPARING,
      STATE_FLYING,
      STATE_FALLING,
      STATE_CRASHED,
      STATE_WALKAWAY
    };

    Ogre::SceneManager* mSceneMgr;
    GameLogic* mLogic;
    Ogre::SceneNode* mNode;
    Ogre::SceneNode* mStartNode;
    Ogre::SceneNode* mSpawnNode;
    Ogre::Entity* mEnt;
    Ogre::Entity* mBallon;
    Ogre::Entity* mStars;
    int mState;
    double mBallonSize;
    Ogre::AnimationState* mAnim;
    double mAccel;
    double mSpeed;
    bool mIsDead;
    int mColour;
    
    // sounds
    audiere::OutputStreamPtr mSoundBlow;
    audiere::OutputStreamPtr mSoundHit;
    audiere::OutputStreamPtr mSoundCrashed;
    audiere::OutputStreamPtr mSoundEscaped;
};

#endif
