#include "needle.h"
#include "constants.h"
#include "mouse.h"
#include <boost/lexical_cast.hpp>
#include "OgreAudiere.h"

using namespace Ogre;
using namespace boost;
using namespace audiere;

const int NUM_SOUNDS_DARTS = 1;

Needle::Needle(SceneManager* sceneMgr, SceneNode* node, Entity* needle, double speed, AudioDevicePtr audioDev)
: mSceneMgr(sceneMgr), mNode(node), mEnt(needle), mIsDead(false)
{
  mNode->attachObject(mEnt);
  // set initial speed.
  mSpeed = mNode->getOrientation() * Vector3::UNIT_X;
  mSpeed.normalise();
  mSpeed *= speed;

  // initialise sound
  mSoundHitGround = OpenSound(audioDev, OpenOgreResourceFile("dart.0.wav"));
  
  // initialise scene query
  mQry = mSceneMgr->createAABBQuery(AxisAlignedBox(), BALLON_FLAG | MOUSE_FLAG);
}

Needle::~Needle()
{
  if (mEnt)
    mSceneMgr->destroyEntity(mEnt);
  if (mNode)
    mSceneMgr->destroySceneNode(mNode->getName());
  mSceneMgr->destroyQuery(mQry);
}

void Needle::update(double passedTime)
{
  // update speed
  mSpeed += Vector3(0.0, GRAVITY*passedTime, 0.0);
  mNode->translate(mSpeed*passedTime);
  // rotate towards speed vector
  Vector3 src = mNode->getOrientation() * Vector3::UNIT_X;
  mNode->rotate(Quaternion (src.getRotationTo(mSpeed)));
  mNode->roll(Radian(Math::HALF_PI));

  if (mNode->getPosition().y <= 0.0)
  {
    // hit the ground.
    if (!mIsDead)
      mSoundHitGround->play();
    mIsDead = true; 
  }

  checkForHit();
}

bool Needle::isDead() const
{
  return mIsDead && !mSoundHitGround->isPlaying();
}


void Needle::checkForHit()
{
  // using an AABB scene query to check for hits with ballons or mice.
  mQry->setBox(mEnt->getWorldBoundingBox());
  mQry->setWorldFragmentType(SceneQuery::WFT_NONE);
  mQry->setQueryMask(BALLON_FLAG | MOUSE_FLAG);
  SceneQueryResult& res = mQry->execute();
  for (SceneQueryResultMovableList::iterator it = res.movables.begin(); it != res.movables.end(); ++it)
  {
    MovableObject* obj = *it;
    if (obj->getQueryFlags() == BALLON_FLAG || obj->getQueryFlags() == MOUSE_FLAG)
    {
      // hit a ballon
      mIsDead = true;
      Mouse* mouse = dynamic_cast<Mouse*>(obj->getUserObject());
      mouse->killBallon();
      break;
    }
  }
}
