#ifndef __OBJECT_H__
#define __OBJECT_H__

/****************************************************
 * OBJECT MANAGEMENT
 * An abstract base class for the game objects as
 * well as a singleton handler for them.
 ****************************************************/

#include <vector>

class Object
{
  public:
    virtual ~Object() { }
    virtual void update(double passedTime) = 0;
    virtual bool isDead() const = 0;
    virtual void show() = 0;
    virtual void hide() = 0;
};


class ObjectManager
{
  private:
    ObjectManager();

    std::vector<Object*> mObjs;

  public:
    static ObjectManager& instance();

    void add(Object* obj);

    void update(double passedTime);

    void hideAll();

    void showAll();

    void deleteAll();

};

#endif
