#ifndef __RANDOM_H__
#define __RANDOM_H__

/***********************************************************
 * RANDOM UTILITY FUNCTIONS
 * These are some convenience wrappers over Boost's random
 * library.
 ***********************************************************/

#include <boost/random/uniform_01.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <ctime>

namespace Random
{
  namespace Impl
  {
    // implementation details
    inline boost::mt19937& rng()
    {
      static boost::mt19937 r (std::time(0));
      return r;
    }

    inline boost::uniform_01<boost::mt19937>& dist()
    {
      static boost::uniform_01<boost::mt19937> d (rng());
      return d;
    }
  }


  /**
   * range returns a random number from the interval [from, to).
   **/
  template<typename NumberType>
  NumberType range(NumberType from, NumberType to)
  {
    return NumberType(from + Impl::dist()()*(to-from));
  }

}

#endif
