#include "score.h"
#include <boost/lexical_cast.hpp>

const double OMEGA = 2.0;
const double AMP = .02;
const double UP_SPEED = 3.0;
const double MAX_HEIGHT = 10.0;

using namespace Ogre;
using namespace boost;
using namespace std;

unsigned int numText = 0;

Score::Score(SceneManager* sceneMgr, SceneNode* node, const string& val)
: mSceneMgr(sceneMgr), mNode(node), mIsDead(false), mTime(0.0), mHeight(0.0)
{
  mText = new MovableText("Score."+lexical_cast<string>(numText++), val, "BlueHighway", 4,
    ColourValue(0x79/255.0,0xfa/255.0,0xff/255.0));
  mNode->attachObject(mText);
}

Score::~Score()
{
  mSceneMgr->destroySceneNode(mNode->getName());
  delete mText;
}

void Score::update(double passedTime)
{
  mTime += passedTime;
  mNode->translate(Vector3::UNIT_Y*UP_SPEED*passedTime + Vector3::UNIT_Z*AMP*OMEGA*Math::Cos(OMEGA*mTime));
  mHeight += UP_SPEED*passedTime;
  if (mHeight >= MAX_HEIGHT)
    mIsDead = true;
}

bool Score::isDead() const
{
  return mIsDead;
}
